/************************************************************************/
/*                                                                      */
/*   Program Name :  vdif2k5                                            */
/*                                                                      */
/*     Vdif2k5 is a general pupose VDIF to K5 file format converter     */
/*                                                                      */
/*     Enviroment variables                                             */
/*           M5DIR      ---   default directory for Mark5 data          */
/*           M5VEX      ---   default directory for Mark5 vex file      */
/*                                                                      */
/*            Ver 1.00  2014-05-28 start development based on m5btok5   */
/*            Ver 1.30  2018-02-09                                      */
/*                                                                      */
/*      Copyright (c) 2014-     T.Kondo All Right Reserved              */
/*                                                                      */
/************************************************************************/
/*

     2014-05-28  -- development started  (only 1ch mode support)
     2016-08-03  -- Ver.1.12 release
     2016-08-03  -- Ver.1.13 debug
     2016-08-08  -- Ver.1.14 debug and  bs_vdif is added as a member of vdifconv_t type variable
     2016-08-08  -- Ver.1.15 further debug
     2016-08-12  -- Ver.1.20 reflect debug in libvdif.c (serious bug regarding time label after July 1st)
     2018-02-09  -- add '-thread n' option to select thread

   how to run
          vdif2k5 vdif_filename [options]
    
     where   vdif_filename -- VDIF data file name
              options (any order)

               -fsampl fmhz   -- channel sampling frequency (MHz) to compulsory set
               -u unit      -- pickup unit (1-4) for 4 ch conversion mode  <= not supported
                               (-1ch option is stronger than this option);
                               (default 4x4ch mode)
			   -1ch vdifch      -- set K5 data to 1ch mode and pick up VDIF ch (starting at 1)
			   -4ch vdifch1 vdifch2 vdifch3 vdifch4
			                  -- set K5 data to 4 ch mode and pick up VDIF 4 channels
			   -2bit mode     -- set 2bit AD decode table
			                         1: offset binary   0,1,2,3 (no conversion) (default)
									 2: Mark V          0,2,1,3
									 3: signed integer  1,0,2,3

               -o k5name     -- K5 file name to be created
                                 (default : see below)
               -d k5dir      -- directory for k5 file out
                                 (default  : mark5 directory)
               -s[offset] soffset
			         +VE : start offset (sec) (default is 0)
					 -VE : absolute value means skip over bytes 
               -k[ashima] [bytesframe] 
                     set header format to Kashima compulsory 
                     bytesframe: +VE bytes/frame set compulsory
               -thread n   -- set pick up thread number (default is 1st thread number)(2018-02-09)
 
  Naming rule for K5 file created (VDIF : original VDIF name)
      Auto 4ch (or 1ch) mode 
         In case of # of VDIF CH > 4     
            VDIF.k5a ---- for unit #1  (usually CH 01-04)
            VDIF.k5b ---- for unit #2  (usually CH 05-08)
            VDIF.k5c ---- for unit #3  (usually CH 09-12)
            VDIF.k5d ---- for unit #4  (usually CH 13-16)
         In case of # of VDIF CH <= 4     
            VDIF.k5 

      1ch mode
            VDIF.k5-NN ---- where NN is channel number 01-16
      4ch mode
            VDIF.k5 


*** sample run parameters ****
G:\chkdata\vdif\kas34full.vdif
D:\chkdata\vdif\test\R218104600a.vdif -d D:\chkdata\vdif\test
D:\chkdata\vdif\test\vdiftest.vdif -4ch 5 6 7 8 -d D:\chkdata\vdif\test
*/

#define PROGRAM_VERSION "vdif2k5 (Ver 1.30  2018-02-09)"

#include "libvdif.h"
//#include "libk5.h"      // common header for IP-VLBI utilities

//#include "libmk5tok5.h"  // library for Mark5 to K5 conversion

/* default conversion information file name set */
//#define INFONAME_DFLT "m5tok5info.txt"   // information file default name
/* environment variable for Mark5 data default directory set */
#define M5DIR "M5DIR" // the environment variable for Mark5 data default directory
/* environment variable for Mark5 vex file default directory set */
#define M5VEX "M5VEX" // the environment variable for Mark5 vex file


/* default parameters for run mode 2 */

#define DFLT_DATA_FILE "vdif.dat"
#define BUFMAX 25000   // maximum read buffer size
#define DFLT_CH_TABLE {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16}  // default CH allocation table

typedef struct vdif2k5runp     // run parameters for mark5 to k5 conversion program
{
    int ptype;   // run parameter type  =1  mode1,  =2 mode 2
    int channel;
    int group;
    int unit;
    int soffset;
    int period;
    int reverse;
	int64_t skip_bytes;   // skip bytes (1st skip over # of bytes)
    int scan;    // scan# for get MODE from VEX file (default=1) 
    int odd;    // =1 4byte integer offset mode in case of 64 track
    int monit;   // =1  inofo_file monitor ON,  =0 no monitoring
    char vdifname[256];   // 2007-07-17  80->256
    //char k5name[256]; // 2007-07-17  80->256
    char k5name[4][256];
    char k5name_given[256]; // 2007-07-17  80->256
    char outdir[256];      // 2007-07-17  80->256
    char info_file[256]; // 2007-07-17  80->256
    char vex_file[256]; // 2007-07-17  80->256
    char sid[4];
    char dflt_pass[4];
    // for mode2 only run parameres are as follows
    int numch;
    int ntrack;
    int adbit;
    int fanout;
    int parity;
    int nrzm;
    int vlba;
    int ibit[32];
    int tot_track;
	int k5numch; 
	int e2bit;
	int vdifch;
	int chtable[16];
    int file_ch_table[4][4]; // channel table for created K5/VSSP file
    int k5nfile;       // 2018-08-08
	int nbitstream;
    int fmode;   // conversion out file mode 1: 1ch mode   2: 4ch mode    3: 4x4ch mode   4:  2x4ch mode
    double fmhz;
    int flag_compulsory;
    int thread_num;   // 2018-02-09
} vdif2k5runp_t;

typedef struct vdifconv     // parameters for functions in vdif2k5
{
    int iwVDIF;   
    int iwK5;
    int iwK5n[4];
    int pickup_ch;
    int *conv2bit;   // pointer to 2bit decoding table
    int k5adbit;
    int k5numch;
    int vdifadbit;
    int k5nfile;       // 2016-08-08
    int nbit;
    int chmax_vdif;
    int bs_vdif;   // bits/stream in VDIF   2016-08-08  
    int chtable[16];
    int nthsample;
    double k5fsample;
    int pup_thread_id;   // pick up thread ID 2018-02-09
} vdifconv_t;

int convVDIF2K5(vdif2k5runp_t *runp);
void conv_vdif2k5_1ch(vdifconv_t *ct);
void conv_vdif2k5_4ch(vdifconv_t *ct);
void conv_vdif2k5_4x4ch(vdifconv_t *ct);
void conv_vdif2k5_4x4ch_fast(vdifconv_t *ct);
void conv_vdif2k5_2x4ch_fast(vdifconv_t *ct);
void conv_vdif2k5_1ch_1ch(vdifconv_t *ct, int nsample);
//void k5name_make_fmode(char *adsname, char *outdir, int fmode, int unit, int channel, int adsnumch, 
//                 char *k5name);
void k5name_make_fmode(vdif2k5runp_t *rp, vdifconv_t *ct, int unit, char *k5name);

int cmnd_get_num(int argc, char *argv[], char *command, char *param, int *num)
// get argument like -command parameter
// search "-command" in argument, if it is exsted, 0 is returned as
// a function value, and next argument is returned as a parameter (ascii).
// if "-command" is not exsited, -1 is returned as a function value.
// if "-command" is not accompanied by real parameter, NULL character
// is returned as param.
// 
{
    int n;
    char cbuf[80];

    param[0]='\0';

    //search command
    //printf("%s\n",command); // debug
    n=2;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        if(strncmp(cbuf,command,strlen(command))==0){  
            n++;
			*num=n;
            if(n > argc) return 0;
            strcpy(param,*(argv+n-1));
			*num++;
            return 0;
        }
        n++;
    }
    return -1;
}

void param_get_vdif2k5(int argc, char *argv[],vdif2k5runp_t *runp)                       
{
    int n,i;
    char cbuf[256];
    int ch_table_dflt[]=DFLT_CH_TABLE;

	// initialize parameters

    /* common parameters */
    runp->ptype=1;
    runp->vdifname[0]='\0';
    //runp->k5name[0]='\0';
    for(n=0; n<4; n++) runp->k5name[n][0]='\0';
    runp->k5name_given[0]='\0';
    runp->soffset=0;
    runp->period=0;
    runp->odd=0;

    /* mode 1 parameters */
    runp->channel=0;
    runp->group=0;
    runp->unit=0;
    runp->reverse=0;
    runp->info_file[0]='\0';
    runp->outdir[0]='\0';
    runp->vex_file[0]='\0';
    runp->sid[0]='\0';
    runp->scan=1;
    runp->monit=0;
	runp->flag_compulsory=0;
	runp->skip_bytes=0;
	runp->k5numch=1; // 
    runp->vdifch=1;

    /* mode 2 parameters */
    runp->numch=0;
    runp->adbit =0;
    //runp->fmhz  =(double)M5SAMPLE;
    runp->fmhz  =0.0;
	runp->nbitstream=0;
    for(n=0; n<32; n++) runp->ibit[n]=0;
    runp->fmode=3;   // default conversion file mode 1: 1ch  2: 4ch  3: 4x4ch  4: 2x4ch
    //for(n=0; n<4; n++) runp->chtable[n]=0;
    for(n=0; n<16; n++) runp->chtable[n]=ch_table_dflt[n];
	//runp->e2bit=2;
	runp->e2bit=1;   // 2016-08-02
    runp->thread_num=-1; // 2018-02-09

    runp->k5nfile=0; //  0 means auto mode  2016-08-08

    /* 1st parameter (vdif file name) get */
    if (argc>=2){
       strcpy(runp->vdifname,*(argv+1));
	   if (strlen(*(argv+1))==1 && runp->vdifname[0]=='0'){  // set default name
          strcpy(runp->vdifname,DFLT_DATA_FILE);
	   }
    }
     /* check VDIFfile name is acompanied by directory or not */
    get_true_path(runp->vdifname, M5DIR, "./",cbuf);
    strcpy(runp->vdifname,cbuf);

    /* command parameters get */
    if(cmnd_get(argc,argv,"-o",cbuf)==0) strcpy(runp->k5name_given,cbuf);  // K5 name
	if(cmnd_get(argc,argv,"-d",cbuf)==0){              // out dir
		strcpy(runp->outdir,cbuf);  
        /* env variable is substituted by real one */
        if((i=substitute_env_path_name(runp->outdir, cbuf))>0){
           strcpy(runp->outdir,cbuf);
        }
	}
//    if(cmnd_get(argc,argv,"-c",cbuf)==0) runp->channel = atoi(cbuf);  // CH
//    if(cmnd_get(argc,argv,"-g",cbuf)==0) runp->group   = atoi(cbuf);  // group
    //if(cmnd_get_p(argc,argv,"-s",cbuf)==0) runp->soffset = atoi(cbuf);  // start offset
    if(cmnd_get(argc,argv,"-p",cbuf)==0) runp->period  = atoi(cbuf);  // period
	if(cmnd_get(argc,argv,"-r",cbuf)==0) runp->reverse=1;  // bit reverse mode
	if(cmnd_get_p(argc,argv,"-si",cbuf)==0) strcpy(runp->sid,cbuf);  // station id
	if(cmnd_get_p(argc,argv,"-scan",cbuf)==0) {  // scan
		  runp->scan=atoi(cbuf);
          if(runp->scan < 1) runp->scan=1;
	}
	if(cmnd_get(argc,argv,"-monit",cbuf)==0) runp->monit=1;  // monitor mode on
    if(cmnd_get(argc,argv,"-fs",cbuf)==0) runp->fmhz  = atof(cbuf);  // VDIF sampling frequency
    if(cmnd_get(argc,argv,"-ad",cbuf)==0) runp->adbit = atoi(cbuf);  // VDIF AD bits
    if(cmnd_get(argc,argv,"-bs",cbuf)==0) runp->nbitstream = atoi(cbuf);  // bit stream
	if(cmnd_get(argc,argv,"-u",cbuf)==0){    // unit define (not supported)
        runp->k5numch=4;
        runp->fmode=2; // 4ch mode
        runp->unit=atoi(cbuf);
    }

	if(cmnd_get(argc,argv,"-1ch",cbuf)==0){    // K5 is 1ch mode
		runp->k5numch=1; 
        runp->fmode=1;     // no more use 2016-08-08
        runp->k5nfile=1;   // 2016-08-08
        runp->vdifch=atoi(cbuf);
	}
	if(cmnd_get_num(argc,argv,"-4ch",cbuf,&n)==0){    // K5 is 4ch mode
		runp->k5numch=4; 
        runp->fmode=2; // 4ch mode  no more use
        runp->k5nfile=1;   // 2016-08-08
        runp->chtable[0]=atoi(cbuf);
        n++;
        if(n > argc) goto exithere;
		runp->chtable[1]=atoi(*(argv + n-1 ));
        n++;
        if(n > argc) goto exithere;
		runp->chtable[2]=atoi(*(argv + n-1 ));
        n++;
        if(n > argc) goto exithere;
		runp->chtable[3]=atoi(*(argv + n-1 ));
	}
exithere:
	if(cmnd_get(argc,argv,"-2bit",cbuf)==0) runp->e2bit = atoi(cbuf);   // 2bit encode mode
	if(cmnd_get_p(argc,argv,"-s",cbuf)==0){
		if(atof(cbuf) >= 0.0){
		    runp->soffset = atoi(cbuf);   // second offset
		} else {
			runp->skip_bytes=-atoi(cbuf);  // skip bytes
		}
	}
    if(cmnd_get(argc,argv,"-k",cbuf)==0 ||cmnd_get(argc,argv,"-K",cbuf)==0 ){ // set compulsory mode
		runp->flag_compulsory=1;
        if(cbuf[0] != '\0'){
            if(cbuf[0] != '-') runp->flag_compulsory=atoi(cbuf);
        }
    }
    if(cmnd_get(argc,argv,"-t",cbuf)==0) runp->thread_num = atoi(cbuf);   // thread number 2018-02-09

    //if(runp->fmode==3) runp->k5numch=4; 
    //if(runp->fmode==4) runp->k5numch=2; 

}

void print_usage(void)
{
    printf("%s\n",PROGRAM_VERSION);
    printf("\n");
    printf("    vdif2k5 vdif_file [options]\n");
    printf("\n");
    printf("where   vdif_file  -- VDIF data file name\n");
    printf("      options (any order)\n");
    printf("        -o k5name     -- K5 file name to be created\n");
    printf("                          (default : see below)\n");
    printf("        -d k5dir      -- directory for k5 file out\n");
    printf("                          (default  : VDIF directory)\n");
    //printf("        -bs nbitsream -- set Mark5B # of bit-stream (1,2,4,8,16,32)\n");
    printf("        -fsampl fmhz  -- channel sampling frequency (MHz) to compulsory set\n");
//    printf("        -u unit       -- pickup unit# (1-4) for 4 ch conversion mode\n");
//    printf("                         (-1ch option is stronger than this option)\n");
//    printf("                         (default 4x4ch mode)\n");
    printf("        -1ch vdifch   -- set K5 data to 1ch mode and \n");
    printf("       	                    pick up VDIF ch (starting at 1)\n");
    printf("        -4ch ch1 ch2 ch3 ch4\n");
    printf("                      -- set K5 data to 4 ch mode and  \n");
    printf(" 	                       pick up VDIF 4 channels\n");
    printf("        -2bit mode    -- set 2bit AD decode table\n");
    printf("                           1: offset binary   0,1,2,3 (no conversion)(default)\n");
    printf("                           2: Mark V standard 0,2,1,3\n");
    printf("                           3: signed integer  1,0,2,3\n");
    printf("        -s[offset] soffset\n");
    printf("                        +VE : start offset (sec) (default is 0.0)\n");
    printf("                        -VE : absolute value means skip over bytes \n");
    printf("        -thread n   -- set pick up thread number (default: 1st thread number)\n");
//    printf("        -k[ashima] [bytesframe]\n");
//    printf("                      -- set header format to Kashima compulsory \n");
//    printf("                         bytesframe: +VE is bytes/frame set compulsory\n");
//    printf("        -s strat_sec  -- offset time from data head (sec)\n");
//    printf("                          (default :  0 )\n");
//    printf("        -p period     -- data period to convert (sec)\n");
//    printf("                          (default : all)\n");
//    printf("        -r            -- reverse track order when fanout =2 or more\n");
//    printf("        -v vex_file   -- VEX file name to be read when \"information\n");
//    printf("                              file creation mode\" is set\n");
//    printf("        -sid stat_id  -- station ID when \"information file creation\n");
//    printf("                              mode\" is set\n");
//    printf("        -scan scan#   -- scan# for MODE get when \"information file\n");
//    printf("                              creation mode\" is set (default : 1)\n");
//    printf("        -subp subpass -- set subpass at information file\n");
//    printf("                              creation mode (default is \"A\")\n");
//    printf("        -odd          -- 32bit (4byte) shift in reading Mark5 data\n");
//    printf("                          (this works only for 64 track mode)\n");
//    printf("        -monit        -- info_file monitor ON\n");
//    printf("\n");
    printf("   Naming rule for K5 file created (VDIF : original VDIF name)\n");
    printf("      4ch mode (auto)\n");
    printf("        VDIF.k5a ---- for unit #1 (usually ch01-04)\n");
    printf("        VDIF.k5b ---- for unit #2 (usually ch05-08)\n");
    printf("        VDIF.k5c ---- for unit #3 (usually ch09-12)\n");
    printf("        VDIF.k5d ---- for unit #4 (usually ch13-16)\n");
    printf("      4ch mode (manually pick-up or 4-ch VDIF)\n");
    printf("        VDIF.k5\n");
    printf("      1ch mode (from multi channel VDIF)\n");
    printf("        VDIF.k5-NN ---- where NN is channel number 01-16\n");
    printf("      1ch mode (from 1ch VDIF)\n");
    printf("        VDIF.k5\n");
//    printf("   Enviroment variables\n");
//    printf("        %s    ---   default path to Mark5 data ( %s )\n",
//        M5DIR,getenv(M5DIR));
//    printf("        %s    ---   default path to Mark5 vex file ( %s )\n",
//        M5VEX,getenv(M5VEX));
    printf("\n");
}

void monit_runp(vdif2k5runp_t *runp)
// run parameter monitor
{
       printf(" ---------- RUN CONDITION ---------\n");
       printf("   vdifname  : %s\n",runp->vdifname);
       printf("   k5name    : %s\n",runp->k5name_given);
       printf("   outdir    : %s\n",runp->outdir);
       printf("   bitstream : %d\n",runp->nbitstream);
       printf("   fmhz      : %f\n",runp->fmhz);
       printf("   adbit     : %d\n",runp->adbit);
       printf("   k5numch   : %d\n",runp->k5numch);
       printf("   vdif_channels : %d %d %d %d\n",runp->chtable[0],
		   runp->chtable[1],runp->chtable[2],runp->chtable[3]);
       printf("   2bit mode : %d\n",runp->e2bit);
//       printf("   infofile  : %s\n",runp->info_file);
//       printf("   channel   : %d\n",runp->channel);
//       printf("   group     : %d\n",runp->group);
//       printf("   soffset   : %d\n",runp->soffset);
//       printf("   period    : %d\n",runp->period);
//       printf("   odd       : %d\n",runp->odd);
//       if(runp->vex_file[0] != '\0')
//          printf("   vex_file  : %s\n",runp->vex_file);
       if(runp->sid[0] != '\0' && runp->sid[0] !='-')
          printf("   station_id: %s\n",runp->sid);

       printf("   oick up thread_id: %d\n",runp->thread_num);
       printf(" -------------------------------------\n");
    printf("\n");
}

int main(int argc, char *argv[])
{
       int i;
       double telapsed;           // to measure processing time
       char fname[256];
       //char fpath[256];
       char program_version[60];
	   time_t tstart, tfinish;    
       //mk5tok5_info_t mk5info;
       //gr_info_t gr; 
       vdif2k5runp_t runp;
       //mk5b_data_t mk5b;

       if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          print_usage();
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
          printf("\n");
          printf("%s\n",PROGRAM_VERSION);
          printf("\n");
          exit(0);
       } 
       if (argc>=2){
          if(strncmp(*(argv+1),"env",3)==0){
          printf("   Enviroment variables\n");
          printf("        %s ---   default directory for Mark5 data\n",M5DIR);
          printf("                      ( %s )\n",getenv(M5DIR));
          printf("        %s ---   default directory for Mark5 vex file\n",M5VEX);
          printf("                      ( %s )\n",getenv(M5VEX));
          printf("\n");
          exit(0);
          }
       }

       strcpy(program_version,PROGRAM_VERSION);

       /* run parameters get */
       param_get_vdif2k5(argc, argv, &runp); 

       /* out directory create if not set */

       if(runp.outdir[0]=='\0'){
          parse_dir_and_name(runp.vdifname, runp.outdir, fname); // debug
          //printf("A %s\n",runp.mk5name);
          //printf("A %s\n",runp.outdir);
          //printf("A %s\n",fname);
       }


       /* Program Name Display */
       printf("************************************************************\n");
       printf("*    VDIF to K5/VSSP32 Data Format Converter               *\n");
       printf("*      %s  by  T.KONDO/NICT    *\n",program_version);
       printf("************************************************************\n");
       printf("\n");

//       monit_runp(&runp);

       tstart= time(NULL);


       i=convVDIF2K5(&runp);

       if(i < 0) goto exitend;

       //printf("  K5 file (%s) has been created.\n\n",runp.k5name);


       tfinish= time(NULL);
       telapsed = difftime(tfinish,tstart);
       printf("Time elapsed (sec) for One obs process is %f\n",telapsed);

exitend: 
       return 0;
}

void vssp32head_make(vdif_file_t *vd, int k5numch, unsigned int *head)
{
/*
     VSSP32 header all make
*/
	int seconds,sfreq,fid,aid,work;
	char cw,cbuf[9];

    /* set sync pattern 1 */
	head[0]=0xFFFFFFFF;
	/* set seconds from 0h UTC */
	seconds=vd->vhead.t.hh*3600+vd->vhead.t.mm*60+vd->vhead.t.ss;
	head[1]=(head[1] & 0xFFFE0000) | (seconds & 0x1FFFF);
	/* set ch flag */
	//if(vd->numch==1){  // 1 ch mode
	if(k5numch==1){  // 1 ch mode
		head[1]=head[1] & 0xFFFDFFFF ;   // set 0
	} else {           // 4 ch mode
		head[1]=head[1] | 0x20000 ;   // set 1
	}
	/* set sampling frequency index */
	sfreq=(int)(vd->fsample/1000);  // (kHz)
    fid=0;
    if( sfreq ==   100) fid=1;
    if( sfreq ==   200) fid=2;
    if( sfreq ==   500) fid=3;
    if( sfreq ==  1000) fid=4;
    if( sfreq ==  2000) fid=5;
    if( sfreq ==  4000) fid=6;
    if( sfreq ==  8000) fid=7;
    if( sfreq == 16000) fid=8;
    if( sfreq == 32000) fid=9;
    if( sfreq == 64000) fid=10;
    if( sfreq == 128000) fid=11;
    if( sfreq == 256000) fid=12;
    if( sfreq == 512000) fid=13;
    if( sfreq == 1024000) fid=14;
    if( sfreq == 2048000) fid=15;
	head[1]=(head[1] & 0xFFC3FFFF) | ((fid & 0xF) << 18);
	/* set AD bit index */
    aid=0;
    if( vd->adbit == 2 ) aid=1;
    if( vd->adbit == 4 ) aid=2;
    if( vd->adbit == 8 ) aid=3;
	head[1]=(head[1] & 0xFF3FFFFF) | ((aid & 0x3) << 22);
    /* set sync pattern 2 */
	head[1]=(head[1] & 0x00FFFFFF) | (0x8C << 24);   // VSSP32    (0x8B for VSSP)

	/* set VSSP32 extention */

	/* set total day */
	work=vd->vhead.t.ddd;
	head[2]=(head[2] & 0xFFFFFE00) | work & 0x1FF;
	/* set year (2 digits) */
	work=vd->vhead.t.year % 100;
	head[2]=(head[2] & 0xFFFF81FF) | ((work & 0x3F) << 9) ;
	/* set eflg to zero */
	head[2]= head[2] & 0xFFFF7FFF ;
	/* set aux field size */
	work=20;
	head[2]=(head[2] & 0xFF00FFFF) | ((work & 0xFF) << 16) ;
	/* set minor version number  */
	work=0;
	head[2]=(head[2] & 0xF0FFFFFF) | ((work & 0xF) << 24) ;
	/* set major version number  */
	work=0;
	head[2]=(head[2] & 0x0FFFFFFF) | ((work & 0xF) << 28) ;

	/* set aux field */

	/* set aux field format number */
	work=1;
	head[3]=(head[3] & 0xFFFFFF00) | (work & 0xFF) ;
	/* set filter frequency */
	work=0;
	head[3]=(head[3] & 0xFFFF00FF) | ((work & 0xFF) << 8) ;
	/* set station id */
	if(vd->vhead.edv==1){ // Kasfima VDIF format
		//head[3]=(head[3] & 0x00FFFFFF) | ((vd->vhead.sid[0] & 0xFF) << 24) ;
		//head[3]=(head[3] & 0xFF00FFFF) | ((vd->vhead.sid[1] & 0xFF) << 16) ;
		head[3]=(head[3] & 0x00FFFFFF) | ((vd->vhead.sid[1] & 0xFF) << 24) ;
		head[3]=(head[3] & 0xFF00FFFF) | ((vd->vhead.sid[0] & 0xFF) << 16) ;
	} else {    // set blank
		cw=' ';
		head[3]=(head[3] & 0x00FFFFFF) | (cw << 24) ;
		head[3]=(head[3] & 0xFF00FFFF) | (cw << 16) ;
	}
	/* set station name */
	if(vd->vhead.edv==1){ // Kasfima VDIF format
		memcpy((char *)head+16, vd->vhead.das, 8);
	} else {
		strcpy(cbuf,"        ");
		memcpy((char *)head+16, cbuf, 8);
	}
	/* set PC host name */
	strcpy(cbuf,"    VDIF");
	memcpy((char *)head+24, cbuf, 8);
}

void vssp32head_sec_update(int seconds, unsigned int *head)
{
/*
     VSSP32 header second updatee
*/
	/* set seconds from 0h UTC */
	head[1]=(head[1] & 0xFFFE0000) | (seconds & 0x1FFFF);
}


int convVDIF2K5(vdif2k5runp_t *runp)
/*
    VDIF to K5 conversion  2014-05-30 
	     (support only 1 ch mode)
*/
{
	unsigned int head[8];   // for VSSP32 header (2 X 32 bits)
    int pcount,icount,opos;
    int i,n,k,m;
    int nf;
    int iwrite;
    int ierr;
	int frame_c;   // total frame numbervcounter 2012-10-17
	int seconds;
    //FILE *fo;
    FILE *fo[4];
    vdif_file_t vd;
    timeval_vdif_t time1st;
    int *obuf[4];
    //int *obuf;
    int nwrite=1024;  // obuf size to write
    vdifconv_t ct;
	//int *conv2bit;
	int conv2bit_1[]={0,1,2,3};   // offset binary (no conversion 2 bit)
    int conv2bit_2[]={0,2,1,3};   //  conversion 2 bit (default)
    int conv2bit_3[]={1,0,2,3};   //  conversion 2 bit singed integer

    printf("  convVDIF2K5 : \n");
    ierr=0;
	frame_c=0; 

	if(runp->e2bit==1){
	   ct.conv2bit=conv2bit_1;
	} else if(runp->e2bit==3){
	   ct.conv2bit=conv2bit_3;
	} else {
	   ct.conv2bit=conv2bit_2;
	}

	memcpy(ct.chtable,runp->chtable,4*sizeof(int));
    ct.pickup_ch=runp->vdifch;    // pick up vdif channel in case of 1 channel mode

    ct.pup_thread_id=runp->thread_num;  // set pick up thread number

    /* array initialize */
    for(n=0; n<4; n++){
        for(i=0; i<4; i++){
            runp->file_ch_table[n][i]=0;
        }
    }

	strcpy(vd.fname,runp->vdifname);
	vd.skip_bytes=runp->skip_bytes;
	vd.flag_compulsory=runp->flag_compulsory;


    /* get VDIF file info by opening and get reading file (then close) */
    //if(vdif_t_ini(&vd,0)<0) goto exit2return;   // VDIF data check with auto mode
    if(vdif_t_ini(&vd,0)<0) exit(0);   // VDIF data check with auto mode
    //if(vdif_t_ini(&vd,1)<0) goto exit2return;  // VDIF data check with 1 sec reading compulsory
    
    //printf("Debug vd.vhead.srate. %d\n",vd.vhead.srate);
    if(vd.vhead.srate == 0){
        if(runp->fmhz <= 0.0){
            printf(" This VDIF file does not include sampling frequency information!\n");
            printf(" Please set sampling frequency using \"-fsampl fmhz\" option.\n");
            exit(0);
        }
    }


    if(vd.fsample==0.0){
        if(runp->fmhz==0.0){
            printf("Warning!  sampling frequency is not included in VDIF headder!!\n");
            printf("Please set sampling frequency using \"-fsampl fMHz\" option!!\n");
            exit(0);
        } else {
            vd.fsample=1.0e6*runp->fmhz;
        }
    } else {
        if(runp->fmhz != 0.0){  // compulsory set sampling frequency  and ndramesec
            vd.fsample=1.0e6*runp->fmhz;
            vd.nframsec=(int)(1.0/((double)vd.frame_bytes*8.0/((double)(vd.adbit*vd.numch)*vd.fsample))+0.5); 
        }
    }

    //printf("Debug vd.nframsec %d\n",vd.nframsec);



    /* set up # of K5 files */

	ct.k5numch=runp->k5numch;
    ct.k5nfile=runp->k5nfile;
    if(ct.k5nfile==0){ // auto mode
        ct.k5numch=vd.numch;
        if(ct.k5numch > 4) ct.k5numch=4;
        ct.k5nfile= (vd.numch-1)/4 + 1;

        if(ct.k5nfile > 4){
            printf(" VDIF # of channels (%d) exceeds limit (16)!\n",vd.numch);
            exit(0);
        }

        /* set up pick up channel table by file */
        for(n=0; n< ct.k5nfile; n++){
           for(i=0; i<4; i++){
               k=n*ct.k5nfile+i +1;
               if(k > vd.numch) k-=vd.numch;
               runp->file_ch_table[n][i]=k;
               ct.chtable[n*ct.k5nfile+i]=k;
           }
        }

    } else {
        // clean up ch table
        if(ct.k5numch == 1){
            ct.chtable[0]=ct.pickup_ch;
            for(n=1; n<16; n++) ct.chtable[n]=0;
        } 
    }


    /* allocate memories */
    //obuf=(int *)malloc(nwrite*sizeof(int));
    for(n=0; n<ct.k5nfile; n++){
       obuf[n]=(int *)malloc(nwrite*sizeof(int));
    }


	time1st=vd.vhead.t;  // save 1st time 

	/* 
	  set reading buffer size to satisfy following condition
	     1. (frame size)/(buffer size) should be an integer number
	*/
	vd.nword=vd.frame_bytes/4;     
	while (vd.nword > BUFMAX) vd.nword/=2;

	if(((vd.frame_bytes/4) % vd.nword) != 0){
		printf(" convVDIF2K5: buffer size does\'nt satisfy the condition! %d %d\n",vd.nword, vd.frame_bytes*4);
	}

	vd.imax=(vd.frame_bytes/4)/vd.nword;   // # ofloops to read entire one frame data
	vd.nsample=vd.nword*32/vd.adbit;  // # of samples in buffer (don't care about # of Chs)

    vd.ubuf  = (u_int32_t *)malloc(vd.nword*sizeof(u_int32_t)); //  data read buffer

	vd.imax_c=0;

	/* VDIF file open for conversion */
	if( (vd.fin  = fopen( vd.fname, "rb" )) == NULL ){
         printf("File (%s) Open Error! \n",vd.fname);
         goto exit2return;
    }
	vd.flag_open=1;
    printf("  VDIF Data File : %s\n",vd.fname);

	ct.k5fsample=vd.fsample;
	ct.k5adbit=vd.adbit;
    ct.vdifadbit=vd.adbit;
	ct.chmax_vdif=vd.numch;
    ct.bs_vdif=vd.adbit*vd.numch;  // bits/stream in VDIF  2016-08-08

	//ct.k5numch=runp->k5numch;


    /* check conversion is possible or not */
    if((32 % ct.bs_vdif) !=0){
        printf("vdif2k5: Warning!! vdif2k5 cannot convert this VDIF file!\n");
        printf("       : bad bits/stream (= %d) condition! \n",ct.bs_vdif);
        exit(0);
    }

	if(vd.flag_vdif_anal==1){ // read pointer move to 1st valid header
		fseek(vd.fin,vd.pos_1st_header,SEEK_SET); // move pointer by K5->bytesec
	}

    /* foward startoffset seconds */
	if(runp->soffset>0){
		printf("  %d second forwading.....",runp->soffset);
	    if(fwdNsecVDIF(runp->soffset, &vd) <0){
           printf("Foward error %d\n",runp->soffset);
           i=fclose( vd.fin ) ;
           goto exit2return;
        }
	    printf("\n");
	    /* get headder info again */
        if(vdif_header_read_mode1(&vd)<0) goto exit2return;
	    time1st=vd.vhead.t;
	}

	//r_thread_id=vd.vhead.thread_id;  // set reading thread ID to 1st one if necessary

    /* output file create */
    //unit=runp->unit;
    printf("\n");
    printf(" Following K5 Data File will be created :\n");
    printf(" =========================================================================\n");
    printf("  Pick_Up_CHs         File Name\n");
    printf(" -------------------------------------------------------------------------\n");
    for(n=0; n<ct.k5nfile; n++){
        //if(ct.k5nfile==1 && ct.chmax_vdif <= 4){
        if(ct.k5nfile==1){
             k5name_make_fmode(runp, &ct, 0,  runp->k5name[n]);
        } else {
             k5name_make_fmode(runp, &ct, n+1,  runp->k5name[n]);
        }
        printf(" ");
        for(i=0; i<4; i++){
            k=n*ct.k5nfile+i;
            //if(runp->file_ch_table[n][i] > 0){
            //    printf(" %2d",runp->file_ch_table[n][i]);
            if(ct.chtable[k] > 0){
                printf(" %2d",ct.chtable[k]);
            } else {
                printf("   ");
            }
        }
        printf("    %s\n",runp->k5name[n]);
        if( (fo[n]  = fopen( runp->k5name[n], "wb" )) == NULL ){
           printf("      File  %s  Open Error! \n",runp->k5name[n]);
           ierr= -2;
           goto exit2return;
        }
    }
    printf(" -------------------------------------------------------------------------\n");
    printf("****************************************************************************\n");

    pcount=0; // converted data period counter (in sec)

    printf("\n  K5 data parameters : ");
    if(ct.k5fsample >= 1.0e6){
        printf("%d MHz %d bit sampling  %d ch data\n",
               //((int)(fsample*1.0e-6)), adbit, numch);
               ((int)(ct.k5fsample*1.0e-6)), ct.k5adbit, ct.k5numch);  // 2016-08-02
    } else {
        printf("%d kHz %d bit sampling  %d ch data\n",
               //(int)(fsample*1.0e-3), adbit, numch);
               (int)(ct.k5fsample*1.0e-3), ct.k5adbit, ct.k5numch);  // 2016-08-02
    }
//    printf("  K5 file (%s) is being created...\n",runp->k5name);

    /* header monitor */
			   // with Versatile ED
    printf(" ==============================================================================\n");
	printf("                                                             EXTENDED DATA\n");
	printf(" HH:MM:SS  FRAME#  I  L  VER  CHS     #Bytes  C AD  TID SID       W1       W2\n");
    printf(" ------------------------------------------------------------------------------\n");


    /* K5 initial header create */
    //vd.numch=k5numch;
    vssp32head_make(&vd,ct.k5numch,head);

    seconds=vd.vhead.t.hh*3600+vd.vhead.t.mm*60+vd.vhead.t.ss;

    ct.nbit=0;
    icount=0;  // 32bit word counter
    opos=0;    // out buffer writing position

    while(1){
		/* second loop */
        vssp32head_sec_update(seconds,head);
		/* K5 header out */
        //iwrite=fwrite(head,sizeof(int),8,fo);
        for(nf=0; nf<ct.k5nfile; nf++) iwrite=fwrite(head,sizeof(int),8,fo[nf]);

		for(n=0; n<vd.nframsec; n++){
read_header_again:
		   /* VDIF header read */
		   i=vdif_header_read_mode2(&vd);

           if(i<0) goto exithere;

           // check thread #
           if(ct.pup_thread_id >=0) { // check thread ID     2018-02-09
               if(vd.vhead.thread_id != ct.pup_thread_id){ // goto next frame
                   fseek(vd.fin, 4*vd.nword, SEEK_CUR);
                   goto read_header_again;
               }
           }

		   if(n==0){
		        printf(" %02d:%02d:%02d %6d",vd.vhead.t.hh,vd.vhead.t.mm,vd.vhead.t.ss, vd.vhead.frame);
	            printf(" %3d %2d  %3d   %2d %10d  %1d %2d %4d  %s %08X %08X\n", 
				vd.vhead.invalid,vd.vhead.legacy,vd.vhead.ver,
					     (int)pow(2.0, vd.vhead.log2_chs),vd.vhead.frame_len64*8,
					     vd.vhead.d_type,vd.vhead.bpsampl+1,vd.vhead.thread_id,vd.vhead.sid,vd.vhead.w1,vd.vhead.w2);
		   }
		   /* VDIF data read */
		   for(k=0; k<vd.imax; k++){
			   vd.wordread = fread( vd.ubuf, sizeof( unsigned int ), vd.nword, vd.fin );
			   if (vd.wordread ==0 ) { // EOF
                   goto exithere; 
		       }

               /* convert to K5 data */
               for(m=0; m<vd.wordread; m++){
                    //iw=vd.ubuf[m];
                    ct.iwVDIF=vd.ubuf[m];

                    if(ct.k5nfile == 1){

                        //iwk5w=obuf[0][opos];
                        ct.iwK5=obuf[0][opos];

                        if(ct.k5numch ==1){

                            /* K5 1ch mode */
                            if(ct.chmax_vdif==1){ // VDIF # of channels
                                obuf[0][opos]=ct.iwVDIF;
                                ct.nbit=0;
                                opos++;
                            } else {
                                 conv_vdif2k5_1ch(&ct);
                                 //obuf[0][opos]=iwk5w;
                                 obuf[0][opos]=ct.iwK5;
                                 if(ct.nbit==32){
                                    ct.nbit=0;
                                    opos++;
                                 }
                            }
                        } else {
                           /* convert to K5 4ch mode */
                             conv_vdif2k5_4ch(&ct);
                             obuf[0][opos]=ct.iwK5;
                             if(ct.nbit==32){
                                ct.nbit=0;
                                opos++;
                             }
                        }

                        if(opos == nwrite){
                           iwrite=fwrite(obuf[0],sizeof(int),nwrite,fo[0]);
                           opos=0;
                        }
                    } else {    // multi K5 file mode

                        for(nf=0; nf<ct.k5nfile; nf++) ct.iwK5n[nf]=obuf[nf][opos];


                        if(ct.k5nfile==4){
                            conv_vdif2k5_4x4ch_fast(&ct);
                        } else if(ct.k5nfile==2){
                            conv_vdif2k5_2x4ch_fast(&ct);
                        } else {
                            printf(" This mode is not supported right now!\n");
                            exit(0);
                        }

                        for(nf=0; nf<ct.k5nfile; nf++) obuf[nf][opos]=ct.iwK5n[nf];

                        if(ct.nbit==32){
                             ct.nbit=0;
                             opos++;
                        }

                        if(opos == nwrite){
                            for(nf=0; nf<ct.k5nfile; nf++) iwrite=fwrite(obuf[nf],sizeof(int),nwrite,fo[nf]);
                            opos=0;
                        }

                    }
               }
		   }
		   frame_c++;  
		}
        // rest write
        if(opos > 0){
             for(nf=0; nf<ct.k5nfile; nf++) iwrite=fwrite(obuf[nf],sizeof(int),opos,fo[nf]);
             opos=0;
        }
		seconds++;

	}
exithere:
    if(vd.flag_open==1) i=fclose( vd.fin) ;
    vd.flag_open=0;
    // rest of data write
    if(opos != 0){
       for(nf=0; nf<ct.k5nfile; nf++) iwrite=fwrite(obuf[nf],sizeof(int),opos,fo[nf]);
    }
    for(nf=0; nf<ct.k5nfile; nf++)i=fclose( fo[nf] ) ;
    printf("  ****************************************************************\n");
	printf("\n");

exit2return:
       //printf("  K5 file (%s) has been created.\n\n",runp->k5name);
	free(vd.ubuf);
    for(nf=0; nf<ct.k5nfile; nf++) free(obuf[nf]);

    if(vd.flag_open==1) i=fclose(vd.fin);

    printf("\n");
    if(ct.k5nfile==1){
    printf(" Following K5 Data File was created :\n");
    } else {
    printf(" Following K5 Data Files were created :\n");
    }
    printf(" =========================================================================\n");
    printf("  Pick_Up_CHs         File Name\n");
    printf(" -------------------------------------------------------------------------\n");
    for(n=0; n<ct.k5nfile; n++){
        printf(" ");
        for(i=0; i<4; i++){
            k=n*ct.k5nfile+i;
            if(ct.chtable[k] > 0){
                printf(" %2d",ct.chtable[k]);
            } else {
                printf("   ");
            }
        }
        printf("    %s\n",runp->k5name[n]);
    }
    printf(" -------------------------------------------------------------------------\n");
    printf("****************************************************************************\n");


    return 0;
}



//void conv_vdif2k5_1ch(int iwADS, int adsch, int *conv2bit, int k5adbit,
//                    int *nbit, int *iwK5)
void conv_vdif2k5_1ch(vdifconv_t *ct)
/*
    VDIF to K5 conversion 1ch mode engine 2016-08-02 

           2016-08-08  debug


  [in]
     iwADS       --- VDIF  a 32bit word data
     adsch       --- VDIF  ch# to be picked up (starting at 1)
                        in case of non-existing ch#, zero data is
                        filled in K5 data
     conv2bit    --- 2bit data conversion table
     k5adbit     --- K5       AD bits  (1 or 2)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 
     iwK5        --- K5 a 32bit word data

  [return]
     nbit        --- already filled out bits in iwK5 (maximum is 32) 
     iwK5        --- K5 a 32bit word data

*/
{
    int m;
    int iw, ik5, nb, ipick;
    int n,vdifch;   // 2016-08-08

    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};


    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};

    int imask8[]={0xFFFFFF00,0xFFFF00FF,0xFF00FFFF,0x00FFFFFF};
    //int chmax=16;
    int chmax;
    //chmax=nbitstream/adbit;

    //iw=iwADS;
    chmax=ct->chmax_vdif;
    //iw=ct->iwVDIF;
    //ik5= *iwK5;
    ik5=ct->iwK5;
    //nb=*nbit;
    nb=ct->nbit;
    vdifch=ct->pickup_ch;




    for(n=0; n< 32/ct->bs_vdif; n++){  // added 2016-08-08
        iw= ct->iwVDIF >> (n*ct->bs_vdif);      // 2016-08-08

        if(ct->pickup_ch > ct->chmax_vdif){
           ipick=0;

        } else if(ct->vdifadbit==8) {   // 8bit
           m=(vdifch-1)*8;
           ipick= (int)(iw >> m) & 0xFF;
        } else if(ct->vdifadbit==4) {   // 4bit
           m=(vdifch-1)*4;
           ipick= (int)(iw >> m) & 0xF;
        } else  if(ct->vdifadbit==2) {  // 2bit
           m=(vdifch-1)*2;
           ipick= (int)(iw >> m) & 0x3;
        } else {     // 1bit
           m=(vdifch-1);
           ipick= (int)(iw >> m) & 0x1;
        }

        if(ct->k5adbit==2){  // convert to K5 2bit mode
           ipick=ct->conv2bit[ipick];
           ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
           nb+=2;
        } else if(ct->k5adbit==4){  // convert to K5 4bit mode 
           ik5 = (ik5 & imask4[nb/4]) | (ipick << nb);
           nb+=4;
        } else if(ct->k5adbit==8){  // convert to K5 8bit mode
           ik5 = (ik5 & imask8[nb/8]) | (ipick << nb);
           nb+=8;
        } else {    // pick up 1bit data
           //ipick >>= 1;   // MSB get
           ik5 = (ik5 & imask[nb]) | (ipick << nb);
           nb++;
        }
    }
    ct->nbit=nb;
    ct->iwK5=ik5;
}


//void conv_vdif2k5_1ch_1ch(int iwADS, int nthsample, int *conv2bit, int k5adbit,
//                    int *nbit, int *iwK5)
void conv_vdif2k5_1ch_1ch(vdifconv_t *ct, int nthsample)
/*
    VDIF (1ch mode) to K5 conversion 1ch mode engine 2014-08-12 

        VDIF format is fixed to as follows

  [in]
     iwADS       --- VDIF   a 32bit word data
     nthsample     --- position of sample data in 32-bit word (starting from 1)
                     total # is 32/k5adbit
     conv2bit    --- 2bit data conversion table
     k5adbit     --- K5       AD bits  (1|2|4|8)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 
     iwK5        --- K5 a 32bit word data

  [return]
     nbit        --- already filled out bits in iwK5 (maximum is 32) 
     iwK5        --- K5 a 32bit word data

*/
{
    int m,n;
    int iw, ik5, nb, ipick;

    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};

    int imask8[]={0xFFFFFF00,0xFFFF00FF,0xFF00FFFF,0x00FFFFFF};

    //int chmax=1;
    //chmax=nbitstream/adbit;


    //iw=ct->iwVDIF;
    ik5= ct->iwK5;
    nb=ct->nbit;

    for(n=0; n< 32/ct->bs_vdif; n++){  // added 2016-08-08
        iw= ct->iwVDIF >> (n*ct->bs_vdif);      // 2016-08-08

        if(ct->pickup_ch > ct->chmax_vdif){
           ipick=0;

        } else if(ct->vdifadbit==8) {   // 8bit
            m=(nthsample-1)*8;
            ipick= (int)(iw >> m) & 0xFF;
        } else if(ct->vdifadbit==4){
            m=(nthsample-1)*4;
            ipick= (int)(iw >> m) & 0xF;
        } else if(ct->vdifadbit==2){   // 2bit
            m=(nthsample-1)*2;
            ipick= (int)(iw >> m) & 0x3;
        } else {   // 1bit
            m=(nthsample-1);
            ipick= (int)(iw >> m) & 0x1;

        }
        
        if(ct->k5adbit==2){  // convert to K5 2bit mode
           ipick=ct->conv2bit[ipick];
           ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
           nb+=2;
        } else if(ct->k5adbit==4){  // convert to K5 4bit mode
           ik5 = (ik5 & imask4[nb/4]) | (ipick << nb);
           nb+=4;
        } else if(ct->k5adbit==8){  // convert to K5 8bit mode
           ik5 = (ik5 & imask8[nb/8]) | (ipick << nb);
           nb+=8;
        } else {    // pick up 1bit data
           //ipick >>= 1;   // MSB get
           ik5 = (ik5 & imask[nb]) | (ipick << nb);
           nb++;
        }
    }
    ct->nbit=nb;
    ct->iwK5=ik5;
}

//void conv_vdif2k5_4ch(int iwADS, int *chtable, int *conv2bit, int k5adbit,
//                    int *nbit, int *iwK5)
void conv_vdif2k5_4ch(vdifconv_t *ct)
/*
    VDIF to K5 conversion 4ch mode engine 2010-06-07 

        VDIF format is fixed to as follows


  [in]
     iwADS       --- VDIF a 32bit word data
     chtable[4]  --- VDIF   ch# to be picked up for K5 4ch data
                        (starting at 1)
                        in case of non-existing ch#, zero data is
                        filled in K5 data
     conv2bit    --- 2bit data conversion table  (only meaningful for 2 bit AD case)
     k5adbit     --- K5       AD bits  (1 or 2 or 4)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
     nbit        --- already filled out bits in iwo (maximum is 32) 
     iwK5        --- K5 a 32bit word data

*/
{
    int m,n;
    int iw, ik5, nb, ipick;
    int k5ch,vdifch;
    //int chmax;
    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};

    int imask8[]={0xFFFFFF00,0xFFFF00FF,0xFF00FFFF,0x00FFFFFF};
    //int chmax=16;

    //iw=ct->iwVDIF;
    ik5= ct->iwK5;
    nb=ct->nbit;
    //chmax=16;                // 2014-08-12
    //if(k5adbit==4) chmax=8;  // 2014-08-12
    //if(k5adbit==8) chmax=4;  // 2014-08-12

    for(n=0; n< 32/ct->bs_vdif; n++){  // added 2016-08-08
        iw= ct->iwVDIF >> (n*ct->bs_vdif);      // 2016-08-08
        for(k5ch=0; k5ch<4; k5ch++){
           vdifch=ct->chtable[k5ch];
           if(vdifch > ct->chmax_vdif){
              ipick=0;
           } else if(ct->vdifadbit==8) {   // 8bit
              m=(vdifch-1)*8;
              ipick= (int)(iw >> m) & 0xFF;
           } else if(ct->vdifadbit==4) {   // 4bit
              m=(vdifch-1)*4;
              ipick= (int)(iw >> m) & 0xF;
           } else  if(ct->vdifadbit==2) {  // 2bit
              m=(vdifch-1)*2;
              ipick= (int)(iw >> m) & 0x3;
           } else {     // 1bit
              m=(vdifch-1);
              ipick= (int)(iw >> m) & 0x1;
           }
           if(ct->k5adbit==2){  // convert to K5 2bit mode
              ipick=ct->conv2bit[ipick];
              ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
              nb+=2;
           } else if(ct->k5adbit==4){  // convert to K5 4bit mode 
              ik5 = (ik5 & imask4[nb/4]) | (ipick << nb);
              nb+=4;
           } else if(ct->k5adbit==8){  // convert to K5 8bit mode
              ik5 = (ik5 & imask8[nb/8]) | (ipick << nb);
              nb+=8;
           } else {    // pick up 1bit data
              //ipick >>= 1;   // MSB get
              ik5 = (ik5 & imask[nb]) | (ipick << nb);
              nb++;
           }
        }
    }
    ct->nbit=nb;
    ct->iwK5=ik5;
}


//void conv_vdif2k5_4x4ch(int iwADS, int *chtable, int *conv2bit, int k5adbit,
//                    int *nbit, int *iwk5n)
void conv_vdif2k5_4x4ch(vdifconv_t *ct)
/*
    VDIF to K5 conversion 4x4ch mode engine 2016-08-02 

        VDIF format is fixed to as follows


  [in]
     iwADS       --- VDIF   a 32bit word data
     chtable[16]  --- VDIF   ch# to be picked up for K5 4x4ch data
                        (starting at 1)
                        in case of non-existing ch#, zero data is
                        filled in K5 data
     conv2bit    --- 2bit data conversion table
     k5adbit     --- K5       AD bits  (1 or 2)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
     nbit        --- already filled out bits in iwo (maximum is 32) 
     iwk5n[4]     --- K5 a 4x32bit word data for 4 unit

*/
{
    int m,n,nf;
    //int iw, ik5, nb, ipick;
    int iw, nb, ipick;
    int k5ch,vdifch;
    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};

    int imask8[]={0xFFFFFF00,0xFFFF00FF,0xFF00FFFF,0x00FFFFFF};

    //int chmax=16;

    //chmax=ct->chmax_vdif;

    //iw=ct->iwVDIF;
    //ik5= *iwK5;
    //nb=ct->nbit;

    for(nf=0; nf<4; nf++){
        nb=ct->nbit;
        for(n=0; n< 32/ct->bs_vdif; n++){  // added 2016-08-08
            iw= ct->iwVDIF >> (n*ct->bs_vdif);      // 2016-08-08
            //nb=nbi;
            for(k5ch=0; k5ch<4; k5ch++){
               vdifch=ct->chtable[nf*4+k5ch];
               if(vdifch > ct->chmax_vdif){
                  ipick=0;
               } else if(ct->vdifadbit==8) {   // 8bit
                  m=(vdifch-1)*8;
                  ipick= (int)(iw >> m) & 0xFF;
               } else if(ct->vdifadbit==4) {   // 4bit
                  m=(vdifch-1)*4;
                  ipick= (int)(iw >> m) & 0xF;
               } else  if(ct->vdifadbit==2) {  // 2bit
                  m=(vdifch-1)*2;
                  ipick= (int)(iw >> m) & 0x3;
               } else {     // 1bit
                  m=(vdifch-1);
                  ipick= (int)(iw >> m) & 0x1;
               }

               if(ct->k5adbit==2){  // convert to K5 2bit mode
                  ipick=ct->conv2bit[ipick];
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask2[nb/2]) | (ipick << nb);
                  nb+=2;
               } else if(ct->k5adbit==4){  // convert to K5 4bit mode 
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask4[nb/4]) | (ipick << nb);
                  nb+=4;
               } else if(ct->k5adbit==8){  // convert to K5 8bit mode
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask8[nb/8]) | (ipick << nb);
                  nb+=8;
               } else {    // pick up 1bit data
                  //ipick >>= 1;   // MSB get
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask[nb]) | (ipick << nb);
                  nb++;
               }
            }
        }
    }
    ct->nbit=nb;
}


//void conv_vdif2k5_4x4ch_fast(int iwADS,  int k5adbit, int *nbit, int *iwk5)
void conv_vdif2k5_4x4ch_fast(vdifconv_t *ct)
/*
    VDIF to K5 conversion 4x4ch mode engine 2016-08-02 
       *** fast version (cahannel allocation and 2bit conversion table is fixed) ****

        VDIF format is fixed to as follows


  [in]
     iwADS       --- VDIF   a 32bit word data
     k5adbit     --- K5       AD bits  (1 or 2)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
     nbit        --- already filled out bits in iwo (maximum is 32) 
     iwk5[4]     --- K5 a 4x32bit word data for 4 unit

*/
{
    int m,n,nf;
    //int iw, ik5, nb, ipick;
    int iw, nb, ipick;
    int k5ch,vdifch;
    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};

    int imask8[]={0xFFFFFF00,0xFFFF00FF,0xFF00FFFF,0x00FFFFFF};

    //int chmax=16;

    //iw=ct->iwVDIF;
    //ik5= *iwK5;
    //nb=ct->nbit;   // start bit per file

    for(nf=0; nf<4; nf++){
        nb=ct->nbit;   // start bit per file
            //nb=nbi;
        for(n=0; n< 32/ct->bs_vdif; n++){  // added 2016-08-08
            iw= ct->iwVDIF >> (n*ct->bs_vdif);      // 2016-08-08
            for(k5ch=0; k5ch<4; k5ch++){
               //adsch=n*4+k5ch;
               vdifch=nf*4+k5ch+1;
               if(vdifch > ct->chmax_vdif){
                  ipick=0;
               } else if(ct->vdifadbit==8){
                  m=(vdifch-1)*8;
                  ipick= (int)(iw >> m) & 0xFF;
               } else if(ct->vdifadbit==4){
                  m=(vdifch-1)*4;
                  ipick= (int)(iw >> m) & 0xF;
               } else if(ct->vdifadbit==2){
                  m=(vdifch-1)*2;
                  ipick= (int)(iw >> m) & 0x3;
               } else {
                  m=(vdifch-1);
                  ipick= (int)(iw >> m) & 0x1;
               }

               if(ct->k5adbit==8){  // convert to K5 8bit mode
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask8[nb/8]) | (ipick << nb);
                  nb+=8;
               } else if(ct->k5adbit==4){  // convert to K5 4bit mode
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask4[nb/4]) | (ipick << nb);
                  nb+=4;
               } else if(ct->k5adbit==2){  // convert to K5 2bit mode
                  //ipick=conv2bit[ipick];
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask2[nb/2]) | (ipick << nb);
                  nb+=2;
               } else {    // pick up 1bit data
                  //ipick >>= 1;   // MSB get
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask[nb]) | (ipick << nb);
                  nb++;
               }
            }
        }
    }
    ct->nbit=nb;
}



//void conv_vdif2k5_2x4ch_fast(int iwADS,  int k5adbit, int *nbit, int *iwk5)
void conv_vdif2k5_2x4ch_fast(vdifconv_t *ct)
/*
    VDIF to K5 conversion 2x4ch mode engine 2016-08-02 
       *** fast version (cahannel allocation is fixed) ****

        VDIF format is fixed to as follows


  [in]
     iwADS       --- VDIF   a 32bit word data
     k5adbit     --- K5       AD bits  (1 or 2 or 4)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
     nbit        --- already filled out bits in iwo (maximum is 32) 
     iwk5[2]     --- K5 a 2x32bit word data for 4 unit

*/
{
    int m,n,nf;
    //int iw, ik5, nb, ipick;
    int iw, nb, ipick;
    int k5ch,vdifch;
    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};

    int imask8[]={0xFFFFFF00,0xFFFF00FF,0xFF00FFFF,0x00FFFFFF};
    //int chmax=8;

    //iw=ct->iwVDIF;
    //ik5= *iwK5;
    //nb=ct->nbit;

    for(nf=0; nf<2; nf++){
        nb=ct->nbit;
        for(n=0; n< 32/ct->bs_vdif; n++){  // added 2016-08-08
            iw= ct->iwVDIF >> (n*ct->bs_vdif);      // 2016-08-08
            for(k5ch=0; k5ch<4; k5ch++){
               //adsch=n*4+k5ch;
               vdifch=nf*4+k5ch+1;
               if(vdifch > ct->chmax_vdif){
                  ipick=0;
               } else if(ct->vdifadbit==8){
                  m=(vdifch-1)*8;
                  ipick= (int)(iw >> m) & 0xFF;
               } else if(ct->vdifadbit==4){
                  m=(vdifch-1)*4;
                  ipick= (int)(iw >> m) & 0xF;
               } else if(ct->vdifadbit==2){
                  m=(vdifch-1)*2;
                  ipick= (int)(iw >> m) & 0x3;
               } else {
                  m=(vdifch-1);
                  ipick= (int)(iw >> m) & 0x1;
               }
               
               if(ct->k5adbit==8){  // convert to K5 8bit mode
                  //ipick=conv2bit[ipick];
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask8[nb/8]) | (ipick << nb);
                  nb+=8;
               } else if(ct->k5adbit==4){  // convert to K5 4bit mode
                  //ipick=conv2bit[ipick];
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask4[nb/4]) | (ipick << nb);
                  nb+=4;
               } else if(ct->k5adbit==2){  // convert to K5 2bit mode
                  //ipick=conv2bit[ipick];
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask2[nb/2]) | (ipick << nb);
                  nb+=2;
               } else {    //  1bit data
                  //ipick >>= 1;   // MSB get
                  ct->iwK5n[nf] = (ct->iwK5n[nf] & imask[nb]) | (ipick << nb);
                  nb++;
               }
            }
        }
    }
    ct->nbit=nb;
}


//void k5name_make_fmode(char *adsname, char *outdir, int fmode, int unit, int channel, int adsnumch, 
//                 char *k5name)
void k5name_make_fmode(vdif2k5runp_t *rp, vdifconv_t *ct, int unit, char *k5name)
{
/*
  Create K5 file name  
         2016-08-08

  Naming rule for K5 file created (ADS : original ADS3000 file name)
    fmode=1
         1ch mode
            ADS.k5-NN ---- where NN is channel number 01-16
         //but if adsnumch=1, name is ADS.k5
    fmode=2
         4ch mode
            ADS.k5[a|b|c|d] --- where a|b|c|d for unit=1|2|3|4
                                or omitted for unit=0
    fmode=3
         4x4ch mode
            ADS.k5a ---- for unit#1  (CH 01-04)
            ADS.k5b ---- for unit#2  (CH 05-08)
            ADS.k5c ---- for unit#3  (CH 09-12)
            ADS.k5d ---- for unit#4  (CH 13-16)
    fmode=4
         2x4ch mode
            ADS.k5a ---- for unit#1  (CH 01-04)
            ADS.k5b ---- for unit#2  (CH 05-08)
*/
    int i,len;
    int flag_remove=0;
    char k5dir[256],dir[256], name[256],fname[256]; 
    char cbuf[256];
    char ag[4][2]={"a","b","c","d"};

    //parse_dir_and_name(adsname, dir, name);
    parse_dir_and_name(rp->vdifname, dir, name);

    /* remove file extension if set so */

    if(flag_remove==1){
        len=strlen(name);

        if(ct->chmax_vdif > 1){
          // find last "." in the name to remove extension
          for(i=len-1; i>=0; i--){
             if(name[i]=='.'){
                 name[i]='\0';
                 break;
             }
          }
        }
    }


    // out directory prepare
    if(rp->outdir[0] == '\0'){
        k5dir[0]='\0';
    } else {
        // converting '\' to '/' if so
        len=strlen(rp->outdir);
        //printf("C %d %s\n",len,outdir);
        for(i=0; i<len; i++){
            if(rp->outdir[i]=='\\'){
                k5dir[i]='/';
            } else {
                k5dir[i]=rp->outdir[i];
            }
        }
        k5dir[len]='\0';
        // check apri dir name accompanied by "/" or not 
        if(k5dir[len-1] != '/') {
           k5dir[len]='/';
           k5dir[len+1]='\0';
        }
    }
    // create k5name
    strcpy(fname,name);
    strcat(fname,".k5");
    //if(fmode == 1){  //  1ch mode
    if(ct->chmax_vdif> 1 && ct->k5numch == 1){  //  1ch mode
        sprintf(cbuf,"-%02d",ct->pickup_ch);
        strcat(fname,cbuf);
    } else {
        if(unit > 0 && unit <5) {
           strcat(fname,ag[unit-1]);
        }
    }
    strcpy(k5name,k5dir);
    strcat(k5name,fname);
}

